<?php

class cDebugger {
	
	private static $instance;
	protected $logFile="./log/error.log";
	protected $mode=2;
	protected $fp;
	protected $debug=false; //toggles debug mode of this class
				                     
	/**
	 * constructor is private
	 * use getInstance() to get a singleton class object
	 */
	protected function __construct(){ 
		try{
			$this->errorLog("Construct");
			$this->open('r');
			$this->close();
		}
		catch(Exception $e){
			$message=$this->exception_handler($e);
			$this->errorLog($message);
			exit($message);
		}	
		
	}
	
	/**
	 * clone is private -> singleton
	 * 
	 */
	private function __clone(){	}
	
	public function __destruct(){ }
	
	/**
	* @info debug method for listing all procedure calls 
	* @access private
	*/
	public function debug($message){
	
		switch ($this->mode){
			case 1: //print mode
				print "{$message}\n";
				break;
			case 2: //log mode
				error_log("{$message}\n", 3, $this->logFile);
				break;	
			default:break;	
		}
		
	}
	
	/**
	* @info debug method for listing all procedure calls 
	* @access private
	*/
	public function showDebugger(){
	
		$message=sprintf("Debugger mode: %s\n\n",$this->mode);			
		$message.=$this->readLog();
		$text = sprintf("<textarea style='width:65em;overflow:auto;white-space:wrap' rows=25>%s</textarea>", $message);
		$text .= sprintf("<form action='%s' method='post' id='debugger' enctype='multipart/form-data'>", e_SELF); 
		$text .= sprintf("<input class='button' style='width:10em' type='submit'  value='%s' id='clear' name='clear' />",REG_TOUR_22);
		$text .= sprintf("</form>");
   		return $text;					
		
	}
		
				
	public function __get($property){
		$this->errorLog("Die Eigenschaft {$property} soll ausgelesen werden.\n");
	}
	
	public function __set($property, $value){
		$this->errorLog("Die Eigenschaft {$property} soll gesetzt werden.\n");
	}
	
	/**
	* @info class internal global exception handler 
	* @parameter: exception
	* @access private
	*/
	protected function exception_handler($exception) {
	  
	  $message  = sprintf("<ul style='margin:1em;'>");
	  $message .= sprintf("<li>Exception: %s</li>", htmlentities($exception->getMessage()));
	  $message .= sprintf("<li>ErrorCode: %s</li>", htmlentities($exception->getCode()));
	  $message .= sprintf("<li>Datei: %s</li>", htmlentities($exception->getFile()));
	  $message .= sprintf("<li>Zeile: %s</li>", htmlentities($exception->getLine()));
	  $message .= sprintf("<li>Trace: %s</li>", htmlentities($exception->getTraceAsString()));
	  $message .= sprintf("</ul>");
	  return $message;
	}
	
	//singleton
	public static function getInstance(){
		
		if(self::$instance == null) self::$instance= new cDebugger();
		return self::$instance;
	}
	
	/**
	* @info errorLog method for listing all errors and exceptions 
	* @access private
	*/
	protected function errorLog($message){
	
		if(!$this->debug)return;
		$message = sprintf("%s: %s", get_class($this),$message);
		$this->debug($message);
		
	}
		
	protected function open($mode='r'){
		
		try{
			
			$this->errorLog("open file {$this->logFile} in mode {$mode}");
			$this->fp = fopen($this->logFile, $mode);
			if($this->fp ===false)
				throw new Exception('Konnte Datei nicht ffnen.');
				
			$this->errorLog("fopen success");	
		}	 
		catch(Exception $e){
			$message=$this->exception_handler($e);
			$this->errorLog($message);
			exit($message);
		}	
	}
	
	protected function lock($mode=LOCK_SH){
		
		try{
			if(flock($this->fp, $mode) ===false)
				throw new Exception('Konnte Datei-Lock nicht erhalten.'); 
			
			$this->errorLog("file lock success");	
		}	 
		catch(Exception $e){
			$message=$this->exception_handler($e);
			$this->errorLog($message);
			exit($message);
		}		
	}
	
	public function clearLog(){
		
		$this->open('w');
		fclose();
		$this->errorLog("clear log");
		
	}	
	
	
	protected function readLog(){
		
		$this->errorLog("read Log file");	
		$this->open();
		$this->lock();
		$text = fread($this->fp, filesize($this->logFile));
		fclose($this->fp);
		
		return $text;
	}
		
	protected function close(){
		
		try{
			$ende=fclose($this->fp);
			if($ende ===false)
				throw new Exception('Konnte Datei nicht schliessen.'); 
				
			$this->errorLog("fclose success");	
				
		}	 
		catch(Exception $e){
			$message=$this->exception_handler($e);
			$this->errorLog($message);
			exit($message);
		}		
		
	}	 
	
	
	
}

?>
