<?php
/*
 ----------------------------------------------------------------------------+
|     e107 website system plugIn
|     http://e107.org
|     jalist@e107.org
|
|     Released under the terms and conditions of the
|     GNU General Public License (http://gnu.org).
|
|     $Source: /cvsroot/e107/e107_0.7/e107_plugins/registration/cTournament.php $
|     $Revision: 1.0 $
|	  $Created on 11.02.2009
|     $Date: 2009/02/11 $
|     $Author: mumm $
+----------------------------------------------------------------------------+
*/
@require_once("../../class2.php");
@require_once("cDebugger.php");  
 //this enables a javascript dhtml calendar for comfortable user input
@require_once(e_HANDLER."calendar/calendar_class.php");
 
 /**
	* @info showing and editing tournament data 
	* @access public
	*/	
 class cTournament {
	
	private static $instance;
	protected $table="turnier";
	protected $cal;
	protected $data;
	protected $debug=false; //toggles debug mode of this class
	protected $error="";
		
	// calendar options go here; see the documentation and/or calendar-setup.js
	protected $calSetUp=array(	'showsTime'      => false,
		                 		'showOthers'     => true,
		                 		'ifFormat'       => '%Y-%m-%d',
		                 		'daFormat'       => '%Y-%m-%d',
		                 		'weekNumbers'    => false);
	// field styles go here
	protected $calInputStyle='color: #840; background-color: #ff8; border: 1px solid #000; text-align: center';
	
	

	
	 /** constructor is private
	 * use getInstance() to get a singleton class object
	 */
	protected function __construct(){	
		
		try{
			$this->debug("Constructor");
			$this->data=$this->getPreferences();
			$this->cal=new DHTML_Calendar(true);
			
		}
		catch(Exception $e){
			$this->exception_handler($e);
			$message=sprintf("%s: %s<p>%s!</p>",REG_REG_99, $e->getMessage(),REG_REG_100);
			exit($message);
		}
		
	}
	
	/**
	 * clone is private -> singleton
	 * 
	 */
	private function __clone(){	}
	
	public function __destruct(){ }
	
	//singleton
	public static function getInstance(){
		
		if(self::$instance == null) self::$instance= new cTournament();
		return self::$instance;
	}
			
	public function __get($property){
		$this->errorLog("Die Eigenschaft {$property} soll ausgelesen werden.\n");
	}
	
	public function __set($property, $value){
		$this->errorLog("Die Eigenschaft {$property} soll gesetzt werden.\n");
	}
	
	/**
	* @info class internal global exception handler 
	* @parameter: exception
	* @access private
	*/
	protected function exception_handler($exception) {
	  
	  $message  = sprintf("Nicht aufgefangene Exception: %s\n", $exception->getMessage());
	  $message .= sprintf("\t\tErrorCode: %s\n", $exception->getCode());
	  $message .= sprintf("\t\tDatei: %s\n", $exception->getFile());
	  $message .= sprintf("\t\tZeile: %s\n", $exception->getLine());
	  $message .= sprintf("\t\tTrace: %s\n\n", $exception->getTraceAsString());
	  $this->errorLog($message);
	}

	
	/**
	* @info debug method for listing content of arrays 
	* @parameter: array
	* @access private
	*/
	protected function debugArray($arr){
	
		//debug
		$content=sprintf("Array size %s, Content...\n",sizeof($arr));
		foreach($arr as $k => $v){
				$content.=sprintf("\t\t[%s]=%s\n", $k, $v);
		}
		$this->debug(sprintf("%s", $content));	
		
	}
	
	/**
	* @info debug method for listing all procedure calls 
	* @parameter: class var debug=true for detailed debug mode
	* @access private
	*/
	protected function debug($message){
	
		if(!$this->debug)return;
		$this->errorLog($message);
		
	}
	
	/**
	* @info errorLog method for listing all errors and exceptions 
	* @access private
	*/
	protected function errorLog($message){
	
		$message = sprintf("%s: %s", get_class($this),$message);
		cDebugger::getInstance()->debug($message);
		
	}
	
		 /** 
	 * return array of current tournament data
	 * access public
	 */
	public function getTournamentData(){
		return $this->data;
	}
	 
	
	
	
	//this enables a javascript dhtml calendar for comfortable user input
	/**
	* @return javascript files to load
	* @info use headerjs() to load JS calendar to head of e 107
	* @access public
	*/
	public function loadCalendar(){
		//returns the file and path of javascript files needed to embed	
		return $this->cal->load_files();
	}
	
	/**
	* @return true if no open tournament is found
	* @info open tournament is dependent on the target date  
	* @access private
	*/
	public function isFinished(){
		
		if (!isset($this->data) || empty($this->data) || !isset($this->data[reg_target]) || empty($this->data[reg_target]))
			return true;
			
		$ar=explode('-',$this->data[reg_target]);
		// result is positiv if target date has passed
		return (mktime() - mktime(0,0,0,$ar[1],$ar[2],$ar[0]) )>0 ; 
		
	}
	
	/**
	* @return true if registration is ongoing
	* @info open registration is dependent on the end date of registration  
	* @access private
	*/
	public function isOpen(){
		
		if (!isset($this->data) || empty($this->data) || !isset($this->data[reg_end]) || empty($this->data[reg_end]))
			return false;
		
		$ar=explode('-',$this->data[reg_end]);
		// result is positiv if end date is still active
		return (mktime(0,0,0,$ar[1],$ar[2],$ar[0]) - mktime())>0 ; 
		
	}
	
		
	/**
	* @return send button dependant on the state of tournament 
	* @access private
	*/
	protected function makeButton(){
		
		$width="10em";
		
		   //target date over
			if(($this->isFinished() && (strlen($this->error)==0)) || (strlen($this->error)>0 && isset($_POST['new'])))
		   		return sprintf("<input class='button' style='width:%s' type='submit'  value='%s' id='%s' name='%s' />",$width, REG_TOUR_1,"new","new");
		   			
		   	return sprintf("<input class='button' style='width:%s' type='submit'  value='%s' id='%s' name='%s' />",$width, REG_TOUR_2,"update","update");		
  			
	}
	
	/**
	* @return user information dependant on the state of tournament 
	* @access private
	*/
	protected function getInfo(){
		
		if(strlen($this->error)>0)return "";
		//target date over
		if($this->isFinished())
			return sprintf("<div style='color:red;text-align:left; margin-bottom:7px;'>%s</div>", 
								REG_TOUR_4);
		//registration ongoing				
  		else if($this->isOpen())
	   		return sprintf("<div style='color:red;text-align:left; margin-bottom:7px;'>
	   						<span style='font-weight:bold'>%s:</span> %s</div>",REG_TOUR_3, REG_TOUR_5);
	   
	   
	   	return sprintf("<div style='color:red;text-align:left; margin-bottom:7px;'>
	   						<span style='font-weight:bold'>%s:</span> %s</div>",REG_TOUR_3,REG_TOUR_6);
   		
   					
	}
	
	/**
	* @return javascript calendar and the dependant input field 
	* @access private
	*/
	protected function getCalendarInput($name){
		
		return sprintf("%s", $this->cal->make_input_field(	$this->calSetUp,
					   // field attributes go here
					      array('style'       => $this->calInputStyle,
					            'name'        => "$name",
					            'value'       => !empty($this->data[$name])?$this->data[$name]:strftime('%Y/%m/%d', strtotime('now')) )
					            )
				);
				
	}
	
	/**
	* @return table for editing tournament data 
	* @access public
	*/
	public function getPreferenceTable(){
		
	
		$text = sprintf("	<div style='text-align:center'>
   							<form action='%s' method='post' id='turnierform' enctype='multipart/form-data'>",	e_SELF);
   							
   		$text .= sprintf("%s",$this->getInfo());
   		$text .= sprintf("<table class='fborder' style='%s'>", ADMIN_WIDTH); 
   							
   		$text .= sprintf("	<tr>
								<td class='forumheader3' style='width:30%%; font-weight:bold; white-space:nowrap;'>%s</td>
								<td class='forumheader3' style='width:70%%;'><input name='reg_tournament' type='text' size='30' maxlength='30' value='%s'></td>
							</tr>",REG_TOUR_7,	$this->data[reg_tournament]);
							
		$text .= sprintf("	<tr>
								<td class='forumheader3' style='width:30%%; font-weight:bold; white-space:nowrap;'>%s</td>
								<td class='forumheader3' style='width:70%%;'>%s</td>
							</tr>",REG_TOUR_8,	$this->getCalendarInput("reg_target"));
							
									
		$text .= sprintf("	<tr>
								<td class='forumheader3' style='width:30%%; font-weight:bold; white-space:nowrap;'>%s</td>
								<td class='forumheader3' style='width:70%%;'>%s</td>
							</tr>",REG_TOUR_9,$this->getCalendarInput("reg_start"));
							
							
		$text .= sprintf("	<tr>
								<td class='forumheader3' style='width:30%%; font-weight:bold; white-space:nowrap;'>%s</td>
								<td class='forumheader3' style='width:70%%;'>%s</td>
							</tr>",REG_TOUR_10,$this->getCalendarInput("reg_end"));
							
		$text .= sprintf("	<tr>
								<td class='forumheader3' style='width:30%%; white-space:nowrap;'>%s</td>
								<td class='forumheader3' style='width:70%%;'><input name='reg_plz' type='text' size='30' maxlength='30' value='%s'></td>
							</tr>",REG_TOUR_11,$this->data[reg_plz]);
							
		$text .= sprintf("	<tr>
								<td class='forumheader3' style='width:30%%; white-space:nowrap;'>%s</td>
								<td class='forumheader3' style='width:70%%;'><input name='reg_city' type='text' size='30' maxlength='30' value='%s'></td>
							</tr>",REG_TOUR_12,$this->data[reg_city]);
							
		$text .= sprintf("	<tr>
								<td class='forumheader3' style='width:30%%; white-space:nowrap;'>%s</td>
								<td class='forumheader3' style='width:70%%;'><input name='reg_adress' type='text' size='30' maxlength='30' value='%s'></td>
							</tr>",REG_TOUR_13,$this->data[reg_adress]);
							
		$text .= sprintf("	<tr>
								<td style='text-align:center' class='forumheader' colspan='2'>%s</td>
							</tr>", $this->makeButton());
									
		$text .= sprintf("	</table></form>");
		$text .= sprintf("	<div style='text-align:left;color:red;margin:15px 0 0 0'>%s</div>",$this->error);		
		$text .= sprintf("	</div>");
		
		return $text;		
	}
	
	/**
	* @return last tournament data 
	* @access public
	*/
	public function getActualTournamentId(){
	
		try {
			global $sql;
			
			$this->debug("DB select {$this->table}");
			$result=$sql->db_Select($this->table, "reg_id" , "ORDER BY reg_id DESC LIMIT 0,1", "");
			
			if(!$result) {
				$error=sprintf("DB Error %s: %s", mysql_errno(), mysql_error());
				if(mysql_errno()>0) throw new Exception ($error);
				$this->debug("Kein Datenbankeintrag gefunden");	
			}
			
			if($row = $sql->db_Fetch())	{	
					return $row[reg_id];
						
			}	
			
			return null;
		
		}	
		catch(Exception $e){
			$this->exception_handler($e);
			$message=sprintf("%s: %s<p>%s!</p>",REG_REG_99, $e->getMessage(),REG_REG_100);
			exit($message);
		}	
	}
	
	/**
	* @return actual tournament name 
	* @access public
	*/
	public function getActualTournament(){
	
		try {
				global $sql;
				
				$this->debug("DB select {$this->table}");
				$result=$sql->db_Select($this->table, "reg_tournament" , "ORDER BY reg_id DESC LIMIT 0,1", "");
				if(!$result) {
					$error=sprintf("DB Error %s: %s", mysql_errno(), mysql_error());
					if(mysql_errno()>0) throw new Exception ($error);
					$this->debug("Kein Datenbankeintrag gefunden");	
				}
				
					if($row = $sql->db_Fetch())	{	
							return $row[reg_tournament];
					}	
				
				return null;
				
		}
		catch(Exception $e){
			$this->exception_handler($e);
			exit();
		}
	}
	
	/**
	* @return last tournament data 
	* @access public
	*/
	protected function getPreferences(){
	
		try{
			global $sql, $tp, $action;
			
			$this->debug("DB select {$this->table}");
			$result=$sql->db_Select($this->table, "*" , "ORDER BY reg_id DESC LIMIT 0,1", "");
			
			if(!$result) {
				$error=sprintf("DB Error %s: %s", mysql_errno(), mysql_error());
				if(mysql_errno()>0) throw new Exception ($error);
				$this->debug("Kein Datenbankeintrag gefunden");	
			}	
			if($row = $sql->db_Fetch())	{	
	
				foreach($row as $k => $v){
					if(strpos($k, "reg_") === 0){
						$this->data[$k] = $v;
					}
				}
				
				//debug
				$this->debug(sprintf("DB result %s",$this->data));
				$this->debugArray($this->data);
				
			}	
			
			return $this->data;
		
		}
		catch(Exception $e){
			$this->exception_handler($e);
			$message=sprintf("%s: %s<p>%s!</p>",REG_REG_99, $e->getMessage(),REG_REG_100);
			exit($message);
		}
		
	}
	
	/**
	* @info validating data  
	* @access private
	*/
	public function validateData(){
	
		$this->error="";
		if(!isset($this->data[reg_tournament]) 	|| empty($this->data[reg_tournament])) 	$this->error .= sprintf("%s.<br>", REG_TOUR_14);
		if(!isset($this->data[reg_target]) 		|| empty($this->data[reg_target])) 		$this->error .= sprintf("%s.<br>", REG_TOUR_15);	
		if(!isset($this->data[reg_start]) 		|| empty($this->data[reg_start])) 		$this->error .= sprintf("%s.<br>", REG_TOUR_16);	
		if(!isset($this->data[reg_end]) 		|| empty($this->data[reg_end])) 		$this->error .= sprintf("%s.<br>", REG_TOUR_17);	
		
		
		if(isset($this->data[reg_start])&& !empty($this->data[reg_start]) && isset($this->data[reg_end])&& !empty($this->data[reg_end]) && isset($this->data[reg_target])&& !empty($this->data[reg_target])) {
		
			$arS=explode('-',$this->data[reg_start]); $arE=explode('-',$this->data[reg_end]); $arT=explode('-',$this->data[reg_target]);
			$epochE=mktime(0,0,0,$arE[1],$arE[2],$arE[0]);
			$epochS=mktime(0,0,0,$arS[1],$arS[2],$arS[0]);
			$epochT=mktime(0,0,0,$arT[1],$arT[2],$arT[0]);
				
			// result is positiv if end date is before start date
			if (($epochS - $epochE ) > 0)  
				$this->error .= sprintf("%s<br>", REG_TOUR_18);
			
			// result is positiv if target date is before start date
			if (($epochS - $epochT ) > 0)  
				$this->error .= sprintf("%s<br>",REG_TOUR_19);
			
			// result is positiv if target date is before end date
			if (($epochE - $epochT ) > 0)  
				$this->error .= sprintf("%s<br>",REG_TOUR_20);		
		}
		
		if(strlen($this->error)>0){
			$errTitle=sprintf("<div style='font-weight:bold; margin-bottom:5px'>%s:</div>",REG_TOUR_21);
			$this->error=$errTitle.$this->error;
		}	
		
		$res=strlen($this->error)==0;
		$this->debug(sprintf("Validating %s",$res?"TRUE":"FALSE"));		
		
		return $res;
		
	}	
	
	/**
	* @info update db using formular input 
	* @access public
	*/
	public function updatePreferences(){
	
	try{
			global $tp, $sql;
				
			while(list($key, $value) = each($_POST)){
				
				foreach($_POST as $k => $v){
					if(strpos($k, "reg_") === 0){
						$this->data[$k] = $tp->toDB($v);
					}
				}
					
			}
			
			$this->data[reg_updater]=USERNAME;
			$this->data[reg_update]=date("Y-m-d H:i:s");
			$this->data[reg_updateip]= $_SERVER['REMOTE_ADDR'];
			
			
			if(!$this->validateData()) return;
					
			$arg=sprintf("reg_tournament='%s', reg_target='%s', reg_start='%s', reg_end='%s', reg_city='%s', reg_plz='%s', reg_adress='%s', reg_updater='%s', reg_update='%s',reg_updateip='%s'  WHERE reg_id=%s",
				$this->data[reg_tournament],
				$this->data[reg_target], 
				$this->data[reg_start],
				$this->data[reg_end],
				$this->data[reg_city],
				$this->data[reg_plz],
				$this->data[reg_adress],
				$this->data[reg_updater],
				$this->data[reg_update],
				$this->data[reg_updateip],
				$this->data[reg_id]);
				
			$this->debug("DB update {$this->table}");
			$this->debug("SQL statement... \n\t{$arg}\n");
			
				
			$result = $sql->db_Update($this->table, $arg);
			if(!$result) {
				$error=sprintf("DB Error %s: %s", mysql_errno(), mysql_error());
				if(mysql_errno()>0) throw new Exception ($error);
				$this->debug("Update Eintrag nicht gefunden");				
			}	
			else $this->debug("Update erfolgreich");
			
			return $result;
		}
		catch(Exception $e){
			$this->exception_handler($e);
			$message=sprintf("%s: %s<p>%s!</p>",REG_REG_99, $e->getMessage(),REG_REG_100);
			exit($message);
		}		
	}
	
	
	/**
	* @info new entry to db using formular input 
	* @access public
	*/	
	public function newPreferences(){
		
		try{
				global $tp, $sql;
				
				
				while(list($key, $value) = each($_POST)){
					
					foreach($_POST as $k => $v){
						if(strpos($k, "reg_") === 0){
							$this->data[$k] = $tp->toDB($v);
						}
					}
				}
							
				unset($this->data[reg_id]);
				$this->data[reg_opener]=USERNAME;
				$this->data[reg_date]=date("Y-m-d H:i:s");
				$this->data[reg_ip]= $_SERVER['REMOTE_ADDR'];
				
				if(!$this->validateData()) return;
				
				//debug
				$this->debug("DB insert {$this->table}");
				$this->debugArray($this->data);
				
				$result = $sql->db_Insert($this->table, $this->data);
				
				if(!$result) {
					$error=sprintf("DB Error %s: %s", mysql_errno(), mysql_error());
					if(mysql_errno()>0) throw new Exception ($error);
				}	
				
				$this->debug("Insert erfolgreich");
				
				return $result;		
				
		}
		catch(Exception $e){
			$this->exception_handler($e);
			$message=sprintf("%s: %s<p>%s!</p>",REG_REG_99, $e->getMessage(),REG_REG_100);
			exit($message);
		}		
		
	}
    
}
?>
